
/**
 * boxes.c: Functions for manipulating lists of boxes.
 * ====================================================================
 * Copyright (c) 2007 Jonathan Coome.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * ====================================================================
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "macros.h"
#include "boxes.h"

void initialise_box_list(box_list_t *list)
{
	list->size = 0;
	list->list = NULL;
}

void add_box_to_list(box_t *box, box_list_t *list)
{
	list->size++;
	list->list = realloc(list->list, list->size * sizeof(box_t));
	
	memcpy(&(list->list[list->size - 1]), box, sizeof(box_t));
}

void free_box_list(box_list_t *list)
{
	free(list->list);
	initialise_box_list(list);
}


void combine_boxes(box_list_t *combined, box_list_t *raw)
{
	int i, j;
	bool is_combined = false;

	for (i = 0; i < raw->size; i++) {
		box_t *box = &raw->list[i];

		for (j = 0; j < combined->size; j++) {
			box_t *container = &combined->list[j];

			if (container->bottom >= box->top - 1 && container->top <= box->bottom - 1 &&
					box->left <= container->right - 1 && box->right >= container->left - 1) {
				container->left   = min(box->left, container->left);
				container->right  = max(box->right, container->right);
				container->top    = min(box->top, container->top);
				container->bottom = max(box->bottom, container->bottom);
				is_combined = true;
			}
		}

		if (!is_combined) add_box_to_list(box, combined);
		is_combined = false;
	}
}
