
/**
 * iobuffer.c: Functions for buffered file reading.
 * ====================================================================
 * Copyright (c) 2007 Jonathan Coome.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * ====================================================================
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include "iobuffer.h"

iobuffer *alloc_buffer(FILE *file)
{
	iobuffer *buf;
	
	buf = calloc(sizeof(iobuffer), 1);
	if (buf == NULL) return NULL;

	buf->file = file;
	return buf;
}

void free_buffer(iobuffer *buf)
{
	fseek(buf->file, buf->position - BUFFER_SIZE, SEEK_CUR);
	free(buf);
}

uint8_t read_byte(iobuffer *buf)
{
	int position;

	if (buf->position == 0) {
		fread(buf->data, BUFFER_SIZE, 1, buf->file);
	}

	position = buf->position;
	buf->position = (position + 1 == BUFFER_SIZE) ? 0 : position + 1;
	return buf->data[position];
}
