
/**
 * pcxfind.c: Where it all comes together.
 * ====================================================================
 * Copyright (c) 2007 Jonathan Coome.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * ====================================================================
 */

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <err.h>
#include <libgen.h>

#include "boxes.h"
#include "pcx.h"
#include "pcxfind.h"

static void parse_image(pcx_image *image, box_list_t *boxes)
{
	int i, j;
	bool in_box = false;
	box_t box;
	box_list_t raw_box_list;

	initialise_box_list(&raw_box_list);

	for (i = 0; i < image->y_size; i++) {
		for (j = 0; j < image->x_size; j++) {
			uint8_t colour = image->image[i * image->x_size + j];

			if (colour != BACKGROUND_COLOUR) {
				if (!in_box) {
					box.left = j;
					box.top = box.bottom = i;
					in_box = true;
				}
				box.right = j;
			} else {
				if (in_box) add_box_to_list(&box, &raw_box_list);
				in_box = false;
			}
		}
	}

	combine_boxes(boxes, &raw_box_list);
	free_box_list(&raw_box_list);
}

int main(int argc, char **argv)
{
	int i;
	FILE *file;
	pcx_image *image;
	box_list_t boxes;

	if (argc < 2) errx(1, "No file specified.");

	if (strcmp(argv[1], "-v") == 0) {
		printf("%s: version %s\n", basename(argv[0]), VERSION);
		return 0;
	}
	
	file = fopen(argv[1], "rb");
	if (file == NULL) err(1, "Could not open '%s'", argv[1]);

	image = pcx_read_image(file);
	if (image == NULL) errx(1, "Could not read image.");

	fclose(file);

	initialise_box_list(&boxes);

	parse_image(image, &boxes);

	pcx_free_image(image);

	for (i = 0; i < boxes.size; i++) {
		box_t *box = &boxes.list[i];
		printf("1 %s  %4d %4d 01 %3d %3d  0  0\n", argv[1], box->left, box->top, box->bottom - box->top + 1, box->right - box->left + 1);
	}

	free_box_list(&boxes);

	return 0;
}
